@@ -70,6 +70,7 @@ enum keyword_numeric_config_field {
     KBF_NOCRSEND,
     KBF_NATTRAVERSAL,
     KBF_DISABLEPORTFLOATING,
+    KBF_FORCE_KEEPALIVE,
     KBF_KEEPALIVE,
     KBF_PLUTORESTARTONCRASH,
     KBF_CRLCHECKINTERVAL,
@@ -79,6 +80,7 @@ enum keyword_numeric_config_field {
     KBF_OPPOENCRYPT,
     KBF_DPDDELAY,
     KBF_DPDTIMEOUT,
+    KBF_METRIC,
     KBF_PHASE2,
     KBF_AUTHBY,
     KBF_KEYEXCHANGE,
