@@ -90,11 +90,12 @@ static const char *const timer_event_name[] = {
 	"EVENT_LOG_DAILY",
 	"EVENT_CRYPTO_FAILED",
 	"EVENT_PENDING_PHASE2",
-	"EVENT_v2_RETRANSMIT"
+	"EVENT_v2_RETRANSMIT",
+	"EVENT_PENDING_DDNS"
     };
 
 enum_names timer_event_names =
-    { EVENT_NULL, EVENT_v2_RETRANSMIT, timer_event_name, NULL };
+    { EVENT_NULL, EVENT_PENDING_DDNS, timer_event_name, NULL };
 
 /* State of exchanges */
 static const char *const state_name[] = {
