@@ -279,6 +279,7 @@ ipsec_klips_init(void)
 
 	error |= ipsec_mast_init_devices();
 
+/* This is no longer needed for >= 2.6.23. We use HAVE_UDP_ENCAP_CONVERT */
 #if defined(NET_26) && defined(CONFIG_IPSEC_NAT_TRAVERSAL)
 	/* register our ESP-UDP handler */
 	if(udp4_register_esp_rcvencap(klips26_rcv_encap
@@ -319,8 +320,10 @@ error_openswan_inet_add_protocol_comp:
 	openswan_inet_del_protocol(&comp_protocol, IPPROTO_COMP);
 #endif /* CONFIG_KLIPS_IPCOMP */
 #endif
+#ifdef CONFIG_KLIPS_AH
 error_openswan_inet_add_protocol_ah:
 	openswan_inet_del_protocol(&ah_protocol, IPPROTO_AH);
+#endif
 error_openswan_inet_add_protocol_esp:
 	openswan_inet_del_protocol(&esp_protocol, IPPROTO_ESP);
 #endif
@@ -345,8 +348,11 @@ error_xmit_state_cache:
 }	
 
 
-/* void */
+#ifdef NET_26
+void
+#else
 int
+#endif
 ipsec_cleanup(void)
 {
 	int error = 0;
@@ -355,11 +361,13 @@ ipsec_cleanup(void)
         ipsec_sysctl_unregister();
 #endif                                                                          
 #if defined(NET_26) && defined(CONFIG_IPSEC_NAT_TRAVERSAL)
-	if(udp4_unregister_esp_rcvencap(klips_old_encap) < 0) {
+	if(udp4_unregister_esp_rcvencap(klips26_rcv_encap, klips_old_encap) < 0) {
 		printk(KERN_ERR "KLIPS: can not unregister klips_rcv_encap function\n");
 	}
 #endif
 
+	error |= ipsec_mast_cleanup_devices();
+
 	KLIPS_PRINT(debug_netlink, /* debug_tunnel & DB_TN_INIT, */
 		    "klips_debug:ipsec_cleanup: "
 		    "calling ipsec_tunnel_cleanup_devices.\n");
@@ -421,10 +429,19 @@ ipsec_cleanup(void)
 
 	prng_final(&ipsec_prng);
 
+#ifdef NET_26
+	if (error)
+		printk("ipsec_cleanup: error %d\n", error);
+#else
 	return error;
+#endif
 }
 
-#ifdef MODULE
+#if defined(MODULE) || LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+#if defined(NET_26) || LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+late_initcall(ipsec_klips_init);
+module_exit(ipsec_cleanup);
+#else
 int
 init_module(void)
 {
@@ -435,7 +452,6 @@ init_module(void)
 	return error;
 }
 
-#ifndef NET_26
 void
 cleanup_module(void)
 {
