@@ -368,7 +368,7 @@ ipsec_spi_get_info(char *buffer,
 			}
 #endif /* CONFIG_KLIPS_IPCOMP */
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 			{
 				char *natttype_name;
 
@@ -399,7 +399,7 @@ ipsec_spi_get_info(char *buffer,
 			}
 #else
 			len += ipsec_snprintf(buffer + len, length-len, " natencap=na");
-#endif /* CONFIG_IPSEC_NAT_TRAVERSAL */
+#endif /* NAT_TRAVERSAL */
 				
 			/* we decrement by one, because this SA has been referenced in order to dump this info */
 			len += ipsec_snprintf(buffer + len,length-len, " refcount=%d",
@@ -550,7 +550,7 @@ ipsec_tncfg_get_info(char *buffer,
 		ipsec_snprintf(name, (ssize_t) sizeof(name), IPSEC_DEV_FORMAT, i);
 		dev = __ipsec_dev_get(name);
 		if(dev) {
-			priv = (struct ipsecpriv *)(dev->priv);
+			priv = (struct ipsecpriv *)netdev_priv(dev);
 			len += ipsec_snprintf(buffer+len, length-len, "%s",
 				       dev->name);
 			if(priv) {
@@ -637,6 +637,8 @@ ipsec_version_get_info(char *buffer,
 
 #ifdef CONFIG_IPSEC_NAT_TRAVERSAL
 unsigned int natt_available = 1;
+#elif defined (HAVE_UDP_ENCAP_CONVERT)
+unsigned int natt_available = 2;
 #else
 unsigned int natt_available = 0;
 #endif
@@ -653,13 +655,7 @@ ipsec_natt_get_info(char *buffer,
 	off_t begin = 0;
 
 	len += ipsec_snprintf(buffer + len,
-			      length-len, "%d\n",
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-			      1
-#else
-			      0
-#endif
-		);
+			      length-len, "%d\n", natt_available);
 
 	*start = buffer + (offset - begin);	/* Start of wanted data */
 	len -= (offset - begin);			/* Start slop */
@@ -947,8 +943,10 @@ ipsec_proc_init()
 				item->read_proc  = it->readthing;
 				item->write_proc = it->writething;
 				item->data       = it->data;
-#ifdef MODULE
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,30)
+# ifdef MODULE
 				item->owner = THIS_MODULE;
+# endif
 #endif
 			} else {
 				error |= 1;
