@@ -502,7 +502,10 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 	ipp  = irs->ipp;
 	ipsp = irs->ipsp;
 	skb  = irs->skb;
-	irs->sa_len = satot(&irs->said, 0, irs->sa, sizeof(irs->sa));
+	if (debug_rcv)
+		irs->sa_len = satot(&irs->said, 0, irs->sa, sizeof(irs->sa));
+	else
+		irs->sa_len = 0;
 	if((ipp->protocol != IPPROTO_IPIP) && 
 	   (ipp->protocol != IPPROTO_ATT_HEARTBEAT)) {  /* AT&T heartbeats to SIG/GIG */
 		KLIPS_PRINT(debug_rcv,
@@ -657,7 +660,7 @@ rcvleave:
 }
 
 /*
- * get all the initial checking and setup done.  Not of this can be off
+ * get all the initial checking and setup done.  None of this can be off
  * loaded by any currently support hardware
  *
  * the following things should be setup when we exit this function.
@@ -681,7 +684,6 @@ rcvleave:
 static enum ipsec_rcv_value
 ipsec_rcv_init(struct ipsec_rcv_state *irs)
 {
-	struct net_device *dev;
 	unsigned char protoc;
 	struct iphdr *ipp;
 	struct net_device_stats *stats = NULL;		/* This device's statistics */
@@ -701,7 +703,6 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv_init: NULL skb.");
 		return IPSEC_RCV_REALLYBAD;
 	}
-	dev = skb->dev;
 
 	if (skb->data == NULL) {
 		KLIPS_PRINT(debug_rcv,
@@ -799,16 +800,8 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 		    "<<< Info -- ");
 	KLIPS_PRINTMORE(debug_rcv && skb->dev, "skb->dev=%s ",
 			skb->dev->name ? skb->dev->name : "NULL");
-	KLIPS_PRINTMORE(debug_rcv && dev, "dev=%s ",
-			dev->name ? dev->name : "NULL");
 	KLIPS_PRINTMORE(debug_rcv, "\n");
 
-	KLIPS_PRINT(debug_rcv && !(skb->dev && dev && (skb->dev == dev)),
-		    "klips_debug:ipsec_rcv: "
-		    "Informational -- **if this happens, find out why** skb->dev:%s is not equal to dev:%s\n",
-		    skb->dev ? (skb->dev->name ? skb->dev->name : "NULL") : "NULL",
-		    dev ? (dev->name ? dev->name : "NULL") : "NULL");
-
 	protoc = ipp->protocol;
 #ifndef NET_21
 	if((!protocol) || (protocol->protocol != protoc)) {
@@ -840,7 +833,7 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 
 		for(i = 0; i <= ipsecdevices_max; i++) {
 			if(ipsecdevices[i] == NULL) continue;
-			prvdev = ipsecdevices[i]->priv;
+			prvdev = netdev_priv(ipsecdevices[i]);
 			
 			if(prvdev == NULL) continue;
 
@@ -851,8 +844,20 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 		}
 
 		if(ipsecdev) {
+			KLIPS_PRINT(debug_rcv && prvdev,
+					"klips_debug:klips26_rcv_encap: "
+					"assigning packet ownership to "
+					"virtual device %s from "
+					"physical device %s.\n",
+				ipsecdev->name, skb->dev->name);
 			skb->dev = ipsecdev;
 		} else {
+			KLIPS_PRINT(debug_rcv,
+					"klips_debug:klips26_rcv_encap: "
+					"assigning packet ownership to "
+					"virtual device mast0 from "
+					"physical device %s.\n",
+				skb->dev->name);
 			skb->dev = ipsec_mast_get_device(0);
 			
 			/* ipsec_mast_get takes the device */
@@ -862,7 +867,10 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 		if(prvdev) {
 			stats = (struct net_device_stats *) &(prvdev->mystats);
 		}
-	} 
+	} else {
+		KLIPS_PRINT(debug_rcv, "klips_debug:klips26_rcv_encap: "
+				"device supplied with skb is NULL\n");
+	}
 
 	if(stats) {
 		stats->rx_packets++;
@@ -1051,7 +1059,7 @@ ipsec_rcv_auth_init(struct ipsec_rcv_state *irs)
 				    irs->sa_len ? irs->sa : " (error)");
 		}
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
                 if (irs->proto == IPPROTO_ESP) {
                         KLIPS_PRINT(debug_rcv,
                                 "klips_debug:ipsec_rcv: "
@@ -1112,7 +1120,7 @@ ipsec_rcv_auth_decap(struct ipsec_rcv_state *irs)
 			irs->ipsp=newipsp;
 
 			/* come back into here with the next transform */
-			irs->next_state = IPSEC_RSM_AUTH_DECAP;
+			irs->next_state = IPSEC_RSM_DECAP_INIT;
 			return IPSEC_RCV_OK;
 		}
 
@@ -1169,7 +1177,7 @@ ipsec_rcv_auth_decap(struct ipsec_rcv_state *irs)
 	 *      if skb->sk is guaranteed to be valid here.
 	 *  2005-04-16: mcr@xelerance.com
 	 */
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 	/*
 	 *
 	 * XXX we should ONLY update pluto if the SA passes all checks,
@@ -1574,20 +1582,6 @@ ipsec_rcv_decap_cont(struct ipsec_rcv_state *irs)
 	}
 #endif /* CONFIG_NETFILTER */
 
-	/* okay, acted on this SA, so free any previous SA, and record a new one */
-	if(irs->ipsp) {
-		struct ipsec_sa *newipsp;
-		newipsp = irs->ipsp->ips_next;
-		if(newipsp) {
-			ipsec_sa_get(newipsp);
-		}
-		if(irs->lastipsp) {
-			ipsec_sa_put(irs->lastipsp);
-		}
-		irs->lastipsp = irs->ipsp;
-		irs->ipsp=newipsp;
-	}
-
 	/* do we need to do more decapsulation */
 	if ((irs->ipp->protocol == IPPROTO_ESP ||
 			irs->ipp->protocol == IPPROTO_AH ||
@@ -1595,7 +1589,7 @@ ipsec_rcv_decap_cont(struct ipsec_rcv_state *irs)
 			irs->ipp->protocol == IPPROTO_COMP ||
 #endif /* CONFIG_KLIPS_IPCOMP */
 			0) && irs->ipsp != NULL) {
-		irs->next_state = IPSEC_RSM_AUTH_DECAP;
+		irs->next_state = IPSEC_RSM_DECAP_INIT;
 	}
 	return IPSEC_RCV_OK;
 }
@@ -1611,15 +1605,29 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 	KLIPS_PRINT(debug_rcv, "klips_debug: %s(st=%d,nxt=%d)\n", __FUNCTION__,
 			irs->state, irs->next_state);
 
+	/* okay, acted on all SA's, so free the last SA, and move to the next */
+	if(irs->ipsp) {
+		struct ipsec_sa *newipsp;
+		newipsp = irs->ipsp->ips_next;
+		if(newipsp) {
+			ipsec_sa_get(newipsp);
+		}
+		if(irs->lastipsp) {
+			ipsec_sa_put(irs->lastipsp);
+		}
+		irs->lastipsp = irs->ipsp;
+		irs->ipsp=newipsp;
+	}
+
 	/* set up for decap loop */
 	ipp  = irs->ipp;
 	ipsp = irs->ipsp;
 	skb = irs->skb;
 
+#ifdef CONFIG_KLIPS_IPCOMP
 	/* if there is an IPCOMP, but we don't have an IPPROTO_COMP,
 	 * then we can just skip it
 	 */
-#ifdef CONFIG_KLIPS_IPCOMP
 	if(irs->ipsp && irs->ipsp->ips_said.proto == IPPROTO_COMP) {
 		struct ipsec_sa *newipsp = NULL;
 		newipsp = irs->ipsp->ips_next;
@@ -1631,10 +1639,11 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 		}
 		irs->lastipsp = irs->ipsp;
 		irs->ipsp=newipsp;
+		irs->sa_len = 0;
 	}
 #endif /* CONFIG_KLIPS_IPCOMP */
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 	if ((irs->natt_type) && (ipp->protocol != IPPROTO_IPIP)) {
 	  /**
 	   * NAT-Traversal and Transport Mode:
@@ -1939,7 +1948,114 @@ ipsec_rcv(struct sk_buff *skb
  */
 int klips26_udp_encap_rcv(struct sock *sk, struct sk_buff *skb)
 {
-	return klips26_rcv_encap(skb, udp_sk(sk)->encap_type);
+	struct udp_sock *up = udp_sk(sk);
+	struct udphdr *uh;
+	struct iphdr *iph;
+	int iphlen, len;
+	int ret;
+
+	__u8 *udpdata;
+	__be32 *udpdata32;
+	__u16 encap_type = up->encap_type;
+
+	/* if this is not encapsulated socket, then just return now */
+	if (!encap_type)
+		return 1;
+
+	/* If this is a paged skb, make sure we pull up
+	* whatever data we need to look at. */
+	len = skb->len - sizeof(struct udphdr);
+	if (!pskb_may_pull(skb, sizeof(struct udphdr) + min(len, 8)))
+		return 1;
+
+	/* Now we can get the pointers */
+	uh = udp_hdr(skb);
+	udpdata = (__u8 *)(uh + 1);
+	udpdata32 = (__be32 *)udpdata;
+
+	switch (encap_type) {
+	default:
+	case UDP_ENCAP_ESPINUDP:
+		KLIPS_PRINT(debug_rcv, "UDP_ENCAP_ESPINUDP_NON_IKE: len=%d 0x%x\n",
+				len, udpdata32[0]);
+		/* Check if this is a keepalive packet.  If so, eat it. */
+		if (len == 1 && udpdata[0] == 0xff) {
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP: keepalive packet detected\n");
+			goto drop;
+		} else if (len > sizeof(struct ip_esp_hdr) && udpdata32[0] != 0) {
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP: ESP IN UDP packet detected\n");
+			/* ESP Packet without Non-ESP header */
+			len = sizeof(struct udphdr);
+		} else {
+			/* Must be an IKE packet.. pass it through */
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP: IKE packet detected\n");
+			return 1;
+		}
+		break;
+	case UDP_ENCAP_ESPINUDP_NON_IKE:
+		KLIPS_PRINT(debug_rcv, "UDP_ENCAP_ESPINUDP_NON_IKE: len=%d 0x%x\n",
+				len, udpdata32[0]);
+		/* Check if this is a keepalive packet.  If so, eat it. */
+		if (len == 1 && udpdata[0] == 0xff) {
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP_NON_IKE: keepalive packet detected\n");
+			goto drop;
+		} else if (len > 2 * sizeof(u32) + sizeof(struct ip_esp_hdr) &&
+				udpdata32[0] == 0 && udpdata32[1] == 0) {
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP_NON_IKE: ESP IN UDP NON IKE packet detected\n");
+			/* ESP Packet with Non-IKE marker */
+			len = sizeof(struct udphdr) + 2 * sizeof(u32);
+		} else {
+			/* Must be an IKE packet.. pass it through */
+			KLIPS_PRINT(debug_rcv,
+					"UDP_ENCAP_ESPINUDP_NON_IKE: IKE packet detected\n");
+			return 1;
+		}
+		break;
+	}
+
+	/* At this point we are sure that this is an ESPinUDP packet,
+	 * so we need to remove 'len' bytes from the packet (the UDP
+	 * header and optional ESP marker bytes) and then modify the
+	 * protocol to ESP, and then call into the transform receiver.
+	 */
+	if (skb_cloned(skb) && pskb_expand_head(skb, 0, 0, GFP_ATOMIC)) {
+		KLIPS_PRINT(debug_rcv, "clone or expand problem\n");
+		goto drop;
+	}
+
+	/* Now we can update and verify the packet length... */
+	iph = ip_hdr(skb);
+	iphlen = iph->ihl << 2;
+	iph->tot_len = htons(ntohs(iph->tot_len) - len);
+	if (skb->len < iphlen + len) {
+		/* packet is too small!?! */
+		KLIPS_PRINT(debug_rcv, "packet too small\n");
+		goto drop;
+	}
+
+	/* pull the data buffer up to the ESP header and set the
+	 * transport header to point to ESP.  Keep UDP on the stack
+	 * for later.
+	 */
+	__skb_pull(skb, len);
+	skb_reset_transport_header(skb);
+
+	/* modify the protocol (it's ESP!) */
+	iph->protocol = IPPROTO_ESP;
+
+	/* process ESP */
+	KLIPS_PRINT(debug_rcv, "starting processing ESP packet\n");
+	ret = klips26_rcv_encap(skb, encap_type);
+	return ret;
+
+drop:
+	kfree_skb(skb);
+	return 0;
 }
 
 int klips26_rcv_encap(struct sk_buff *skb, __u16 encap_type)
@@ -1971,16 +2087,9 @@ int klips26_rcv_encap(struct sk_buff *skb, __u16 encap_type)
 		goto rcvleave;
 	}
 
-	/* XXX fudge it so that all nat-t stuff comes from ipsec0    */
-	/*     eventually, the SA itself will determine which device
-	 *     it comes from
-	 */ 
-	{
-	  skb->dev = ipsec_get_device(0);
-	}
 	irs->hard_header_len = skb->dev->hard_header_len;
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 	switch(encap_type) {
 	case UDP_ENCAP_ESPINUDP:
 	  irs->natt_type = ESPINUDP_WITH_NON_ESP;
@@ -1997,7 +2106,7 @@ int klips26_rcv_encap(struct sk_buff *skb, __u16 encap_type)
 	  }
 	  goto rcvleave;
 	}
-#endif /* NAT_T */
+#endif /* NAT_TRAVERSAL */
 
 	irs->skb = skb;
 
@@ -2045,10 +2154,7 @@ ipsec_rcv_state_cache_init (void)
 
         spin_lock_init(&irs_cache_lock);
 #ifdef HAVE_KMEM_CACHE_MACRO
-        /* irs_cache_allocator = KMEM_CACHE(ipsec_irs,0); */
-        irs_cache_allocator = kmem_cache_create ("ipsec_irs",
-                sizeof (struct ipsec_rcv_state), 0,
-                0, NULL);
+        irs_cache_allocator = KMEM_CACHE(ipsec_rcv_state,0);
 #else
         irs_cache_allocator = kmem_cache_create ("ipsec_irs",
                 sizeof (struct ipsec_rcv_state), 0,
@@ -2112,7 +2218,7 @@ ipsec_rcv_state_new (void)
         irs->said.proto = 0;
 
         irs->hard_header_len = 0;
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
         irs->natt_type = 0;
         irs->natt_len = 0;
 #endif
