@@ -35,6 +35,7 @@
 #endif /* MALLOC_SLAB */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
+#include <linux/file.h> 
 #include <linux/interrupt.h> /* mark_bh */
 
 #include <net/tcp.h>
@@ -49,14 +50,10 @@
 
 #include <openswan.h>
 
-#ifdef NET_21
 # include <linux/in6.h>
 # define IS_MYADDR RTN_LOCAL
 # include <net/dst.h>
-# undef dev_kfree_skb
-# define dev_kfree_skb(a,b) kfree_skb(a)
 # define PHYSDEV_TYPE
-#endif /* NET_21 */
 
 #ifndef NETDEV_TX_BUSY
 # ifdef NETDEV_XMIT_CN
@@ -86,6 +83,7 @@
 #include "openswan/ipsec_sa.h"
 #include "openswan/ipsec_tunnel.h"
 #include "openswan/ipsec_xmit.h"
+#include "openswan/ipsec_rcv.h"
 #include "openswan/ipsec_ipe4.h"
 #include "openswan/ipsec_ah.h"
 #include "openswan/ipsec_esp.h"
@@ -104,7 +102,7 @@ static __u32 zeroes[64];
 DEBUG_NO_STATIC int
 ipsec_tunnel_open(struct net_device *dev)
 {
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 	
 	/*
 	 * Can't open until attached.
@@ -571,7 +569,7 @@ ipsec_tunnel_xsm_complete(
 		return;
 	}
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 	stat = ipsec_nat_encap(ixs);
 	if(stat != IPSEC_XMIT_OK) {
 		goto cleanup;
@@ -610,7 +608,7 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	if (! ixs) {
 		struct ipsecpriv *prv;
 		struct net_device_stats *stats;
-		prv = dev->priv;
+		prv = netdev_priv(dev);
 		stats = (struct net_device_stats *) &(prv->mystats);
 		stats->tx_dropped++;
 		goto alloc_error;
@@ -661,7 +659,7 @@ alloc_error:
 DEBUG_NO_STATIC struct net_device_stats *
 ipsec_tunnel_get_stats(struct net_device *dev)
 {
-	return &(((struct ipsecpriv *)(dev->priv))->mystats);
+	return &(((struct ipsecpriv *)netdev_priv(dev))->mystats);
 }
 
 /*
@@ -673,7 +671,7 @@ DEBUG_NO_STATIC int
 ipsec_tunnel_hard_header(struct sk_buff *skb, struct net_device *dev,
 	unsigned short type, const void *daddr, const void *saddr, unsigned len)
 {
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 	struct net_device *tmp;
 	int ret;
 	struct net_device_stats *stats;	/* This device's statistics */
@@ -805,7 +803,7 @@ ipsec_tunnel_rebuild_header(void *buff, struct net_device *dev,
 			unsigned long raddr, struct sk_buff *skb)
 #endif /* NET_21 */
 {
-	struct ipsecpriv *prv = skb->dev->priv;
+	struct ipsecpriv *prv = netdev_priv(skb->dev);
 	struct net_device *tmp;
 	int ret;
 	struct net_device_stats *stats;	/* This device's statistics */
@@ -895,7 +893,7 @@ ipsec_tunnel_rebuild_header(void *buff, struct net_device *dev,
 DEBUG_NO_STATIC int
 ipsec_tunnel_set_mac_address(struct net_device *dev, void *addr)
 {
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 	
 	struct net_device_stats *stats;	/* This device's statistics */
 	
@@ -925,7 +923,7 @@ ipsec_tunnel_set_mac_address(struct net_device *dev, void *addr)
 		return -ENODEV;
 	}
 
-	if(!prv->set_mac_address) {
+	if(!prv->dev) {
 		KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
 			    "klips_debug:ipsec_tunnel_set_mac_address: "
 			    "physical device has been detached, cannot set - skb->dev=%s->NULL\n",
@@ -946,7 +944,7 @@ DEBUG_NO_STATIC void
 ipsec_tunnel_cache_bind(struct hh_cache **hhp, struct net_device *dev,
 				 unsigned short htype, __u32 daddr)
 {
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = (struct ipsecpriv *) netdev_priv(dev);
 	
 	struct net_device_stats *stats;	/* This device's statistics */
 	
@@ -998,7 +996,7 @@ DEBUG_NO_STATIC void
 ipsec_tunnel_cache_update(struct hh_cache *hh, const struct net_device *dev,
 				const unsigned char *  haddr)
 {
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 	
 	struct net_device_stats *stats;	/* This device's statistics */
 	
@@ -1099,7 +1097,7 @@ DEBUG_NO_STATIC int
 ipsec_tunnel_attach(struct net_device *dev, struct net_device *physdev)
 {
         int i;
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 
 	if(dev == NULL) {
 		KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
@@ -1116,6 +1114,7 @@ ipsec_tunnel_attach(struct net_device *dev, struct net_device *physdev)
 		return -ENODATA;
 	}
 
+	dev->set_mac_address = ipsec_tunnel_set_mac_address;
 	prv->dev = physdev;
 	prv->hard_start_xmit = physdev->hard_start_xmit;
 	prv->get_stats = physdev->get_stats;
@@ -1198,7 +1197,7 @@ DEBUG_NO_STATIC int
 ipsec_tunnel_detach(struct net_device *dev)
 {
         int i;
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 
 	if(dev == NULL) {
 		KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
@@ -1287,7 +1286,7 @@ ipsec_tunnel_clear(void)
 	for(i = 0; i < IPSEC_NUM_IF; i++) {
    	        ipsecdev = ipsecdevices[i];
 		if(ipsecdev != NULL) {
-			if((prv = (struct ipsecpriv *)(ipsecdev->priv))) {
+			if((prv = (struct ipsecpriv *)netdev_priv(ipsecdev))) {
 				prvdev = (struct net_device *)(prv->dev);
 				if(prvdev) {
 					KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
@@ -1316,7 +1315,7 @@ DEBUG_NO_STATIC int
 ipsec_tunnel_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
 {
 	struct ipsectunnelconf *cf = (struct ipsectunnelconf *)&ifr->ifr_data;
-	struct ipsecpriv *prv = dev->priv;
+	struct ipsecpriv *prv = netdev_priv(dev);
 	struct net_device *them; /* physical device */
 #ifdef CONFIG_IP_ALIAS
 	char *colon;
@@ -1402,30 +1401,35 @@ ipsec_tunnel_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
 #endif /* KLIPS */
 
 #ifdef HAVE_UDP_ENCAP_CONVERT
+	/*
+	 * we get two int args in ifr_data, the socket fd (0)
+	 * and the type of encapsulation needed (1).
+	 */
 	case IPSEC_UDP_ENCAP_CONVERT:
 	{
-		unsigned int *socknum =(unsigned int *)&ifr->ifr_data;
-		struct socket *sock;
-		int err, fput_needed;
+		unsigned int *ifp =(unsigned int *)&ifr->ifr_data;
+		const struct socket *sock;
+		int err = 0;
 
-		/* that's a static function in socket.c 
- 		 * sock = sockfd_lookup_light(*socknum, &err, &fput_needed); */
-		sock = sockfd_lookup(*socknum, &err);
+ 		/* translate # to socket structure */
+		sock = sockfd_lookup(ifp[0], &err);
 		if (!sock)
 			goto encap_out;
 
 		/* check that it's a UDP socket */
-		udp_sk(sk)->encap_type = UDP_ENCAP_ESPINUDP_NON_IKE;
-		udp_sk(sk)->encap_rcv  = klips26_udp_encap_rcv;
+		udp_sk(sock->sk)->encap_type = ifp[1];
+		udp_sk(sock->sk)->encap_rcv  = klips26_udp_encap_rcv;
 
 		KLIPS_PRINT(debug_tunnel
-			    , "UDP socket: %u set to NON-IKE encap mode\n"
-			    , socknum);
+			    , "UDP socket: %u set to %s (0x%x) encap mode\n"
+			    , ifp[0]
+				, ifp[1] == UDP_ENCAP_ESPINUDP_NON_IKE ?
+					"UDP_ENCAP_ESPINUDP_NON_IKE" : "UDP_ENCAP_ESPINUDP_NON_ESP"
+				, ifp[1]
+				);
 		       
 		err = 0;
 
-	encap_output:
-		fput_light(sock->file, fput_needed);
 	encap_out:
 		return err;
 	}
@@ -1510,7 +1514,7 @@ ipsec_device_event(struct notifier_block *unused, unsigned long event, void *ptr
 			ipsec_dev = ipsecdevices[i];
 
 			if(ipsec_dev) {
-				priv = (struct ipsecpriv *)(ipsec_dev->priv);
+				priv = (struct ipsecpriv *)netdev_priv(ipsec_dev);
 				if(priv) {
 					;
 					if(((struct net_device *)(priv->dev)) == dev) {
@@ -1616,10 +1620,12 @@ ipsec_tunnel_init(struct net_device *dev)
 	dev->hard_start_xmit	= ipsec_tunnel_start_xmit;
 	dev->get_stats		= ipsec_tunnel_get_stats;
 
+#ifndef alloc_netdev
 	dev->priv = kmalloc(sizeof(struct ipsecpriv), GFP_KERNEL);
 	if (dev->priv == NULL)
 		return -ENOMEM;
-	memset((caddr_t)(dev->priv), 0, sizeof(struct ipsecpriv));
+#endif
+	memset(netdev_priv(dev), 0, sizeof(struct ipsecpriv));
 
 	for(i = 0; i < sizeof(zeroes); i++) {
 		((__u8*)(zeroes))[i] = 0;
@@ -1711,7 +1717,7 @@ ipsec_tunnel_createnum(int ifnum)
 
 	sprintf(name, IPSEC_DEV_FORMAT, ifnum);
 #ifdef alloc_netdev
-	dev_ipsec = alloc_netdev(0, name, ipsec_tunnel_netdev_setup);
+	dev_ipsec = alloc_netdev(sizeof(struct ipsecpriv), name, ipsec_tunnel_netdev_setup);
 #else
 	dev_ipsec = (struct net_device*)kmalloc(sizeof(struct net_device), GFP_KERNEL);
 #endif
@@ -1819,8 +1825,8 @@ ipsec_tunnel_deletenum(int vifnum)
 	dev_ipsec->name=NULL;
 #endif /* !NETDEV_23 */
 	kfree(dev_ipsec->priv);
-#endif /* alloc_netdev */
 	dev_ipsec->priv=NULL;
+#endif /* alloc_netdev */
 
 	return 0;
 }
@@ -1872,8 +1878,8 @@ ipsec_tunnel_cleanup_devices(void)
 		dev_ipsec->name=NULL;
 #endif /* !NETDEV_23 */
 		kfree(dev_ipsec->priv);
-#endif /* alloc_netdev */
 		dev_ipsec->priv=NULL;
+#endif /* alloc_netdev */
 	}
 	return error;
 }
@@ -1976,7 +1982,7 @@ ipsec_xmit_state_new (void)
 		ixs->ips.ips_ident_s.data = NULL;
 		ixs->ips.ips_ident_d.data = NULL;
 		ixs->outgoing_said.proto = 0;
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 		ixs->natt_type = 0, ixs->natt_head = 0;
 		ixs->natt_sport = 0, ixs->natt_dport = 0;
 #endif
