@@ -254,10 +254,13 @@ receive_ping(int afamily, int s, int reply, int natt)
 	printf("received %d(%s) packet from %s/%d of len: %d\n",
 	       ih.isa_xchg, xchg_name, buf, ntohs(rport), n);
 
+	u_int32_t tmp_ic, tmp_rc;
+	memcpy(&tmp_ic, ih.isa_icookie, sizeof(u_int32_t));
+	memcpy(&tmp_rc, ih.isa_rcookie, sizeof(u_int32_t));
 	printf("\trcookie=%08x_%08x icookie=%08x_%08x msgid=%08x\n",
-	       *(u_int32_t *)(ih.isa_icookie), 
+	       tmp_ic,
 	       *(u_int32_t *)(ih.isa_icookie+4), 
-	       *(u_int32_t *)(ih.isa_rcookie), 
+	       tmp_rc,
 	       *(u_int32_t *)(ih.isa_rcookie+4),
 	       ih.isa_msgid);
 	printf("\tnp=%03d  version=%d.%d    xchg=%s(%d)\n",
