@@ -1,5 +1,9 @@
 /* information about connections between hosts and clients
+ *
  * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2003-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -151,8 +155,8 @@ update_host_pairs(struct connection *c)
     ip_address new_addr;
     char *dnshostname;
 
-    d = c->host_pair->connections;
     p = c->host_pair;
+    d = p ? p->connections : NULL;
 
     if (d == NULL
     	|| p == NULL
@@ -838,10 +842,16 @@ load_end_certificate(const char *filename, struct end *dst)
 	    bool valid_cert = FALSE;
 	    
 	    /* load cert from file */
+#ifdef HAVE_LIBNSS
+	    valid_cert = load_cert_from_nss(FALSE, filename, TRUE,"host cert", &cert);
+#else
 	    valid_cert = load_host_cert(FALSE, filename, &cert, TRUE);
+#endif
 	    if(!valid_cert) {
 		whack_log(RC_FATAL, "can not load certificate file %s\n"
 			  , filename);
+		/* clear the ID, we're expecting it via %fromcert */
+		dst->id.kind = ID_NONE;
 		return;
 	    }
 	}
@@ -959,7 +969,11 @@ extract_end(struct end *dst, const struct whack_end *src, const char *which)
 	/* certificate is a blob */
 	dst->cert.forced = TRUE;
 	dst->cert.type = src->certtype;
+#ifdef HAVE_LIBNSS
+	load_cert_from_nss(TRUE, src->cert, TRUE, "forced cert", &dst->cert);
+#else
 	load_cert(TRUE, src->cert, TRUE, "forced cert", &dst->cert);
+#endif
     } else {
 	/* load local end certificate and extract ID, if any */
 	load_end_certificate(src->cert, dst);
@@ -1323,6 +1337,8 @@ add_connection(const struct whack_message *wm)
         c->dpd_timeout = wm->dpd_timeout;
         c->dpd_action = wm->dpd_action;
 
+	c->metric = wm->metric;
+
 	c->forceencaps = wm->forceencaps;
 
 	c->addr_family = wm->addr_family;
@@ -2089,6 +2105,14 @@ route_owner(struct connection *c
 		    continue;
 		if (src->that.port != srd->that.port)
 		    continue;
+
+		/* with old eroutes/routing, we could not do this. This
+		 * allows a host with two IP's to talk to 1 oter host
+		 * with both IP's using two different tunnels.
+		 */
+		if (!sameaddr(&src->this.host_addr, &srd->this.host_addr))
+			continue;
+
 		passert(oriented(*d));
 		if (srd->routing > best_routing)
 		{
