@@ -11,7 +11,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: connections.h,v 1.104 2005/08/25 02:24:45 paul Exp $
  */
 
 /* There are two kinds of connections:
@@ -258,6 +257,10 @@ struct connection {
     ip_address modecfg_dns2;
     ip_address modecfg_wins1;
     ip_address modecfg_wins2;
+#endif
+    u_int8_t metric;              /* metric for tunnel routes */
+#ifdef HAVE_STATSD
+    u_int32_t statsval;			/* track what we have told statsd */
 #endif
 };
 
@@ -415,6 +418,10 @@ extern void host_pair_enqueue_pending(const struct connection *c
 			       , struct pending **pnext);
 struct pending **host_pair_first_pending(const struct connection *c);
 
+#ifdef DYNAMICDNS
+void connection_check_ddns(void);
+#endif
+
 void connection_check_phase2(void);
 void init_connections(void);
 
@@ -435,6 +442,10 @@ extern struct connection *unoriented_connections;
 
 extern void update_host_pairs(struct connection *c);
 
+#ifdef HAVE_LIBNSS
+extern void load_authcerts_from_nss(const char *type, u_char auth_flags);
+#endif
+
 /*
  * Local Variables:
  * c-basic-offset:4
