@@ -1,7 +1,9 @@
 /* IPsec DOI and Oakley resolution routines
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2002  D. Hugh Redelmeier.
- * Copyright (C) 2003-2006  Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2008 Michael C. Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,12 +15,10 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- *
  * Modifications to use OCF interface written by
  * Daniel Djamaludin <danield@cyberguard.com>
- * Copyright (C) 2004-2005 Intel Corporation.  All Rights Reserved.
+ * Copyright (C) 2004-2005 Intel Corporation.
  *
- * RCSID $Id: ipsec_doi.c,v 1.304.2.11 2006/04/16 02:24:19 mcr Exp $
  */
 
 #include <stdio.h>
@@ -79,6 +79,8 @@
 #include "ikev1.h"
 #include "ikev1_continuations.h"
 
+#include "oswcrypto.h"
+
 #ifdef XAUTH
 #include "xauth.h"
 #endif
@@ -94,10 +96,6 @@
 
 #include "tpm/tpm.h"
 
-#ifdef HAVE_OCF
-#include "ocf_pk.h"
-#endif
-
 /* Initiate an Oakley Main Mode exchange.
  * --> HDR;SA
  * Note: this is not called from demux.c
@@ -143,7 +141,7 @@ main_outI1(int whack_sock
 	openswan_log("initiating Main Mode to replace #%lu", predecessor->st_serialno);
 
     /* set up reply */
-    init_pbs(&md.reply, reply_buffer, sizeof(reply_buffer), "reply packet");
+    init_pbs(&reply_stream, reply_buffer, sizeof(reply_buffer), "reply packet");
 
     /* HDR out */
     {
@@ -156,7 +154,7 @@ main_outI1(int whack_sock
 	memcpy(hdr.isa_icookie, st->st_icookie, COOKIE_SIZE);
 	/* R-cookie, flags and MessageID are left zero */
 
-	if (!out_struct(&hdr, &isakmp_hdr_desc, &md.reply, &md.rbody))
+	if (!out_struct(&hdr, &isakmp_hdr_desc, &reply_stream, &md.rbody))
 	{
 	    reset_cur_state();
 	    return STF_INTERNAL_ERROR;
@@ -239,11 +237,11 @@ main_outI1(int whack_sock
 #endif
 
     close_message(&md.rbody);
-    close_output_pbs(&md.reply);
+    close_output_pbs(&reply_stream);
 
     /* let TCL hack it before we mark the length and copy it */
     TCLCALLOUT("avoidEmitting", st, st->st_connection, &md);
-    clonetochunk(st->st_tpacket, md.reply.start, pbs_offset(&md.reply)
+    clonetochunk(st->st_tpacket, reply_stream.start, pbs_offset(&reply_stream)
 	, "reply packet for main_outI1");
 
     /* Transmit */
@@ -280,7 +278,6 @@ main_outI1(int whack_sock
  * This will *not* generate other hash payloads (eg. Phase II or Quick Mode,
  * New Group Mode, or ISAKMP Informational Exchanges).
  * If the hashi argument is TRUE, generate HASH_I; if FALSE generate HASH_R.
- * If hashus argument is TRUE, we're generating a hash for our end.
  * See RFC2409 IKE 5.
  *
  * Generating the SIG_I and SIG_R for DSS is an odd perversion of this:
@@ -309,34 +306,59 @@ main_mode_hash_body(struct state *st
 
     if (hashi)
     {
+#ifdef HAVE_LIBNSS
+        hmac_update_chunk(ctx, st->st_gi);
+        hmac_update_chunk(ctx, st->st_gr);
+        hmac_update(ctx, st->st_icookie, COOKIE_SIZE);
+        hmac_update(ctx, st->st_rcookie, COOKIE_SIZE);
+#else
 	hash_update_chunk(ctx, st->st_gi);
 	hash_update_chunk(ctx, st->st_gr);
 	hash_update(ctx, st->st_icookie, COOKIE_SIZE);
 	hash_update(ctx, st->st_rcookie, COOKIE_SIZE);
+#endif
     }
     else
     {
+#ifdef HAVE_LIBNSS
+        hmac_update_chunk(ctx, st->st_gr);
+        hmac_update_chunk(ctx, st->st_gi);
+        hmac_update(ctx, st->st_rcookie, COOKIE_SIZE);
+        hmac_update(ctx, st->st_icookie, COOKIE_SIZE);	
+#else
 	hash_update_chunk(ctx, st->st_gr);
 	hash_update_chunk(ctx, st->st_gi);
 	hash_update(ctx, st->st_rcookie, COOKIE_SIZE);
 	hash_update(ctx, st->st_icookie, COOKIE_SIZE);
+#endif
     }
 
     DBG(DBG_CRYPT, DBG_log("hashing %lu bytes of SA"
 	, (unsigned long) (st->st_p1isa.len - sizeof(struct isakmp_generic))));
 
     /* SA_b */
+#ifdef HAVE_LIBNSS
+    hmac_update(ctx, st->st_p1isa.ptr + sizeof(struct isakmp_generic)
+        , st->st_p1isa.len - sizeof(struct isakmp_generic));
+#else
     hash_update(ctx, st->st_p1isa.ptr + sizeof(struct isakmp_generic)
 	, st->st_p1isa.len - sizeof(struct isakmp_generic));
+#endif
 
     /* Hash identification payload, without generic payload header.
      * We used to reconstruct ID Payload for this purpose, but now
      * we use the bytes as they appear on the wire to avoid
      * "spelling problems".
      */
+#ifdef HAVE_LIBNSS
+    hmac_update(ctx
+        , idpl->start + sizeof(struct isakmp_generic)
+        , pbs_offset(idpl) - sizeof(struct isakmp_generic));
+#else
     hash_update(ctx
 	, idpl->start + sizeof(struct isakmp_generic)
 	, pbs_offset(idpl) - sizeof(struct isakmp_generic));
+#endif
 
 #   undef hash_update_chunk
 #   undef hash_update
@@ -351,7 +373,11 @@ main_mode_hash(struct state *st
     struct hmac_ctx ctx;
 
     hmac_init_chunk(&ctx, st->st_oakley.prf_hasher, st->st_skeyid);
+#ifdef HAVE_LIBNSS
+    main_mode_hash_body(st, hashi, idpl, &ctx, NULL);
+#else
     main_mode_hash_body(st, hashi, idpl, &ctx.hash_ctx, ctx.h->hash_update);
+#endif
     hmac_final(hash_val, &ctx);
     return ctx.hmac_digest_len;
 }
@@ -451,6 +477,9 @@ RSA_sign_hash(struct connection *c
  * 1	SIG length doesn't match key length -- wrong key
  * 2-8	malformed ECB after decryption -- probably wrong key
  * 9	decrypted hash != computed hash -- probably correct key
+ * 10   NSS error
+ * 11   NSS error
+ * 12   NSS error
  *
  * Although the math should be the same for generating and checking signatures,
  * it is not: the knowledge of the private key allows more efficient (i.e.
@@ -474,13 +503,19 @@ try_RSA_signature_v1(const u_char hash_val[MAX_DIGEST_LEN], size_t hash_len
 	return "1" "SIG length does not match public key length";
     }
 
+#ifdef HAVE_LIBNSS
+    err_t ugh = RSA_signature_verify_nss (k,hash_val,hash_len,sig_val,sig_len);
+    if(ugh!=NULL) {
+	return ugh;
+    }
+#else
     /* actual exponentiation; see PKCS#1 v2.0 5.1 */
     {
 	chunk_t temp_s;
 	MP_INT c;
 
 	n_to_mpz(&c, sig_val, sig_len);
-	cryptodev.mod_exp(&c, &c, &k->e, &k->n);
+	oswcrypto.mod_exp(&c, &c, &k->e, &k->n);
 
 	temp_s = mpz_to_n(&c, sig_len);	/* back to octets */
 	memcpy(s, temp_s.ptr, sig_len);
@@ -554,6 +589,7 @@ try_RSA_signature_v1(const u_char hash_val[MAX_DIGEST_LEN], size_t hash_len
 	/* XXX notification: INVALID_HASH_INFORMATION */
 	return "9" "authentication failure: received SIG does not match computed HASH, but message is well-formed";
     }
+#endif
 
     /* Success: copy successful key into state.
      * There might be an old one if we previously aborted this
@@ -705,6 +741,19 @@ main_inI1_outR1(struct msg_digest *md)
 	numvidtosend++;
     }
 #endif
+
+#ifdef HAVE_LIBNSS
+       if(PK11_IsFIPS())
+       {
+#define SEND_PLUTO_VID 0
+	}
+	else
+	{
+#define SEND_PLUTO_VID 1
+	}
+
+#endif
+
 #if SEND_PLUTO_VID || defined(openpgp_peer)
     numvidtosend++;
 #endif
@@ -855,7 +904,7 @@ main_inI1_outR1(struct msg_digest *md)
     }
 
     /* parse_isakmp_sa also spits out a winning SA into our reply,
-     * so we have to build our md->reply and emit HDR before calling it.
+     * so we have to build our reply_stream and emit HDR before calling it.
      */
 
     /* HDR out.
@@ -868,7 +917,7 @@ main_inI1_outR1(struct msg_digest *md)
 	r_hdr.isa_flags &= ~ISAKMP_FLAG_COMMIT;	/* we won't ever turn on this bit */
 	memcpy(r_hdr.isa_rcookie, st->st_rcookie, COOKIE_SIZE);
 	r_hdr.isa_np = ISAKMP_NEXT_SA;
-	if (!out_struct(&r_hdr, &isakmp_hdr_desc, &md->reply, &md->rbody))
+	if (!out_struct(&r_hdr, &isakmp_hdr_desc, &reply_stream, &md->rbody))
 	    return STF_INTERNAL_ERROR;
     }
 
@@ -975,6 +1024,14 @@ main_inR1_outI2_continue(struct pluto_crypto_req_cont *pcrc
     DBG(DBG_CONTROLMORE
 	, DBG_log("main inR1_outI2: calculated ke+nonce, sending I2"));
 
+    if (st == NULL) {
+	loglog(RC_LOG_SERIOUS, "%s: Request was disconnected from state",
+		__FUNCTION__);
+	if (ke->md)
+	    release_md(ke->md);
+	return;
+    }
+
     /* XXX should check out ugh */
     passert(ugh == NULL);
     passert(cur_state == NULL);
@@ -1075,7 +1132,7 @@ main_inR1_outI2_tail(struct pluto_crypto_req_cont *pcrc
     struct state *const st = md->st;
 
     /**************** build output packet HDR;KE;Ni ****************/
-    init_pbs(&md->reply, reply_buffer, sizeof(reply_buffer), "reply packet");
+    init_pbs(&reply_stream, reply_buffer, sizeof(reply_buffer), "reply packet");
 
     /* HDR out.
      * We can't leave this to comm_handle() because the isa_np
@@ -1158,6 +1215,14 @@ main_inI2_outR2_continue(struct pluto_crypto_req_cont *pcrc
     DBG(DBG_CONTROLMORE
 	, DBG_log("main inI2_outR2: calculated ke+nonce, sending R2"));
 
+    if (st == NULL) {
+	loglog(RC_LOG_SERIOUS, "%s: Request was disconnected from state",
+		__FUNCTION__);
+	if (ke->md)
+	    release_md(ke->md);
+	return;
+    }
+
     /* XXX should check out ugh */
     passert(ugh == NULL);
     passert(cur_state == NULL);
@@ -1299,13 +1364,14 @@ main_inI2_outR2_tail(struct pluto_crypto_req_cont *pcrc
 	&& !has_preloaded_public_key(st)
 	&& st->st_connection->spd.that.ca.ptr != NULL;
 
-    /* HDR out done */
+    /* HDR out */
+    echo_hdr(md, FALSE, ISAKMP_NEXT_KE);
 
     /* KE out */
     if (!ship_KE(st, r, &st->st_gr
 		 , &md->rbody, ISAKMP_NEXT_NONCE))
 	{
-	    abort();
+	    osw_abort();
 	return STF_INTERNAL_ERROR;
 	}
 
@@ -1341,8 +1407,8 @@ main_inI2_outR2_tail(struct pluto_crypto_req_cont *pcrc
     }
 #else
     /* Nr out */
-    if (!ship_nonce(&st->st_nr
-		    , &md->rbody, r
+    if (!ship_nonce(&st->st_nr, r
+		    , &md->rbody
 		    , (send_cr)? ISAKMP_NEXT_CR : ISAKMP_NEXT_NONE
 		    , "Nr"))
 	return STF_INTERNAL_ERROR;
@@ -1691,6 +1757,14 @@ main_inR2_outI3_cryptotail(struct pluto_crypto_req_cont *pcrc
   
   DBG(DBG_CONTROLMORE
       , DBG_log("main inR2_outI3: calculated DH, sending R1"));
+
+  if (st == NULL) {
+      loglog(RC_LOG_SERIOUS, "%s: Request was disconnected from state",
+	      __FUNCTION__);
+      if (dh->md)
+          release_md(dh->md);
+      return;
+  }
   
   passert(cur_state == NULL);
   passert(st != NULL);
@@ -1781,7 +1855,7 @@ oakley_id_and_auth(struct msg_digest *md
     /* ID Payload in.
      * Note: this may switch the connection being used!
      */
-    if (!decode_peer_id(md, initiator, aggrmode))
+    if (!aggrmode && !decode_peer_id(md, initiator, FALSE))
 	return STF_FAIL + INVALID_ID_INFORMATION;
 
     /* Hash the ID Payload.
@@ -2265,7 +2339,6 @@ send_isakmp_notification(struct state *st
 			 , u_int16_t type, const void *data, size_t len)
 {
     msgid_t msgid;
-    pb_stream reply;
     pb_stream rbody;
     u_char old_new_iv[MAX_DIGEST_LEN];
     u_char old_iv[MAX_DIGEST_LEN];
@@ -2275,7 +2348,7 @@ send_isakmp_notification(struct state *st
         
     msgid = generate_msgid(st);
     
-    init_pbs(&reply, reply_buffer, sizeof(reply_buffer), "ISAKMP notify");
+    init_pbs(&reply_stream, reply_buffer, sizeof(reply_buffer), "ISAKMP notify");
     
     /* HDR* */
     {
@@ -2287,7 +2360,7 @@ send_isakmp_notification(struct state *st
         hdr.isa_flags = ISAKMP_FLAG_ENCRYPTION;
         memcpy(hdr.isa_icookie, st->st_icookie, COOKIE_SIZE);
         memcpy(hdr.isa_rcookie, st->st_rcookie, COOKIE_SIZE);
-        if (!out_struct(&hdr, &isakmp_hdr_desc, &reply, &rbody))
+        if (!out_struct(&hdr, &isakmp_hdr_desc, &reply_stream, &rbody))
             impossible();
     }
     /* HASH -- create and note space to be filled later */
@@ -2352,7 +2425,7 @@ send_isakmp_notification(struct state *st
     {  
         chunk_t saved_tpacket = st->st_tpacket;
 
-        setchunk(st->st_tpacket, reply.start, pbs_offset(&reply));
+        setchunk(st->st_tpacket, reply_stream.start, pbs_offset(&reply_stream));
         send_packet(st, "ISAKMP notify", TRUE);
         st->st_tpacket = saved_tpacket;
     }       
