@@ -2,6 +2,8 @@
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  * Copyright (C) 2003-2006  Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,12 +15,10 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- *
  * Modifications to use OCF interface written by
  * Daniel Djamaludin <danield@cyberguard.com>
- * Copyright (C) 2004-2005 Intel Corporation.  All Rights Reserved.
+ * Copyright (C) 2004-2005 Intel Corporation.
  *
- * RCSID $Id: ipsec_doi.c,v 1.304.2.11 2006/04/16 02:24:19 mcr Exp $
  */
 
 #include <stdio.h>
@@ -93,10 +93,6 @@
 
 #include "tpm/tpm.h"
 
-#ifdef HAVE_OCF
-#include "ocf_pk.h"
-#endif
-
 /* Pluto's Vendor ID
  *
  * Note: it is a NUL-terminated ASCII string, but NUL won't go on the wire.
@@ -171,7 +167,7 @@ echo_hdr(struct msg_digest *md, bool enc, u_int8_t np)
 	r_hdr.isa_flags |= ISAKMP_FLAG_ENCRYPTION;
     /* some day, we may have to set r_hdr.isa_version */
     r_hdr.isa_np = np;
-    if (!out_struct(&r_hdr, &isakmp_hdr_desc, &md->reply, &md->rbody)) {
+    if (!out_struct(&r_hdr, &isakmp_hdr_desc, &reply_stream, &md->rbody)) {
 	impossible();	/* surely must have room and be well-formed */
     }
 }
@@ -194,13 +190,23 @@ unpack_KE(struct state *st
 	
 	clonetochunk(*g, wire_chunk_ptr(kn, &(kn->gi))
 		     , kn->gi.len, "saved gi value");
+#ifdef HAVE_LIBNSS
+	DBG(DBG_CRYPT, DBG_log("saving DH priv (local secret) and pub key into state struc"));
+	clonetochunk(st->st_sec_chunk
+		     , wire_chunk_ptr(kn, &(kn->secret))
+		     , kn->secret.len, "pointer to DH private key (secret)");
 
+	clonetochunk(st->pubk
+		     , wire_chunk_ptr(kn, &(kn->pubk))
+		     , kn->pubk.len, "pointer to DH public key");
+#else
 	n_to_mpz(&st->st_sec
 		 , wire_chunk_ptr(kn, &(kn->secret))
 		 , kn->secret.len);
 	clonetochunk(st->st_sec_chunk
 		     , wire_chunk_ptr(kn, &(kn->secret))
 		     , kn->secret.len, "long term secret");
+#endif
     }
 }
 
@@ -312,6 +318,7 @@ static initiator_function *pick_initiator(struct connection *c UNUSED, lset_t po
 	return ikev2parent_outI1;
 
     } else {
+	openswan_log("Neither IKEv1 nor IKEv2 allowed");
 	/*
 	 * tried IKEv2, if allowed, and failed,
 	 * and tried IKEv1, if allowed, and got nowhere.
@@ -399,8 +406,10 @@ ipsecdoi_replace(struct state *st
 
 	initiator_function *initiator = pick_initiator(c, policy);
 	passert(!HAS_IPSEC_POLICY(policy));
-	(void) initiator(whack_sock, st->st_connection, st, policy
-			 , try, st->st_import);
+	if(initiator) {
+	    (void) initiator(whack_sock, st->st_connection, st, policy
+			     , try, st->st_import);
+	}
     }
     else
     {
@@ -590,6 +599,13 @@ decode_peer_id(struct msg_digest *md, bool initiator, bool aggrmode)
 	return FALSE;
     }
 
+    /*
+     * For interop with SoftRemote/aggressive mode we need to remember some
+     * things for checking the hash
+     */
+    st->st_peeridentity_protocol = id->isaid_doi_specific_a;
+    st->st_peeridentity_port = id->isaid_doi_specific_b;
+
     {
 	char buf[IDTOA_BUF];
 
