@@ -1,7 +1,15 @@
 /* netlink interface to the kernel's IPsec mechanism
  *
- * Copyright (C) 2003 Herbert Xu.
- * Copyright (C) 2006 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003,2008 Herbert Xu.
+ * Copyright (C) 2006-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2006 Ken Bantoft <ken@xelerance.com>
+ * Copyright (C) 2007 Bart Trojanowski <bart@jukie.net>
+ * Copyright (C) 2007 Ilia Sotnikov
+ * Copyright (C) 2009 Carsten Schlote <c.schlote@konzeptpark.de>
+ * Copyright (C) 2008 Andreas Steffen
+ * Copyright (C) 2008 Neil Horman <nhorman@redhat.com>
+ * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2006-2009 Paul Wouters <paul@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,8 +20,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * RCSID $Id: kernel_netlink.c,v 1.31 2005/08/14 21:58:09 mcr Exp $
  */
 
 #if defined(linux) && defined(NETKEY_SUPPORT)
@@ -58,6 +64,10 @@
 #include <security/pam_appl.h>
 #endif
 
+#ifndef DEFAULT_UPDOWN
+# define DEFAULT_UPDOWN "ipsec _updown"
+#endif
+
 extern const struct pfkey_proto_info null_proto_info[2];
 
 static const struct pfkey_proto_info broad_proto_info[2] = { 
@@ -1408,9 +1418,9 @@ netlink_shunt_eroute(struct connection *c
                    , enum pluto_sadb_operations op 
 		   , const char *opname)
 {
-	loglog(RC_COMMENT, "request to %s a %s policy with netkey kernel --- experimental"
+    DBG(DBG_CONTROL, DBG_log("request to %s a %s policy with netkey kernel --- experimental"
 		, opname
-		, enum_name(&routing_story, rt_kind));
+		, enum_name(&routing_story, rt_kind)));
 
     /* We are constructing a special SAID for the eroute.
      * The destination doesn't seem to matter, but the family does.
@@ -1776,6 +1786,57 @@ netlink_get_sa(const struct kernel_sa *sa, u_int *bytes)
     return TRUE;
 }
 
+static bool
+netkey_do_command(struct connection *c, struct spd_route *sr
+		 , const char *verb, struct state *st)
+{
+    char cmd[2048];     /* arbitrary limit on shell command length */
+    char common_shell_out_str[2048];
+    const char *verb_suffix;
+
+    /* figure out which verb suffix applies */
+    {
+        const char *hs, *cs;
+
+        switch (addrtypeof(&sr->this.host_addr))
+        {
+            case AF_INET:
+                hs = "-host";
+                cs = "-client";
+                break;
+            case AF_INET6:
+                hs = "-host-v6";
+                cs = "-client-v6";
+                break;
+            default:
+                loglog(RC_LOG_SERIOUS, "unknown address family");
+                return FALSE;
+        }
+        verb_suffix = subnetisaddr(&sr->this.client, &sr->this.host_addr)
+            ? hs : cs;
+    }
+
+    if(fmt_common_shell_out(common_shell_out_str, sizeof(common_shell_out_str), c, sr, st)==-1) {
+	loglog(RC_LOG_SERIOUS, "%s%s command too long!", verb, verb_suffix);
+	return FALSE;
+    }
+
+    if (-1 == snprintf(cmd, sizeof(cmd)
+		       , "2>&1 "   /* capture stderr along with stdout */
+		       "PLUTO_VERB='%s%s' "
+		       "%s"        /* other stuff   */
+		       "%s"        /* actual script */
+		       , verb, verb_suffix
+		       , common_shell_out_str
+		       , sr->this.updown == NULL? DEFAULT_UPDOWN : sr->this.updown))
+    {
+	loglog(RC_LOG_SERIOUS, "%s%s command too long!", verb, verb_suffix);
+	return FALSE;
+    }
+
+    return invoke_command(verb, verb_suffix, cmd);
+}
+
 const struct kernel_ops netkey_kernel_ops = {
     kern_name: "netkey",
     type: USE_NETKEY,
@@ -1795,7 +1856,7 @@ const struct kernel_ops netkey_kernel_ops = {
     process_queue: NULL,
     grp_sa: NULL,
     get_spi: netlink_get_spi,
-    docommand: do_command_linux,
+    docommand: netkey_do_command,
     process_ifaces: netlink_process_raw_ifaces,
 
     /* XXX these needed to be added */
