@@ -24,11 +24,15 @@
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>     /* used only if MSG_NOSIGNAL not defined */
+#include <net/if.h>
+#include <sys/ioctl.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 #include <openswan/pfkeyv2.h>
 #include <openswan/pfkey.h>
+#include <openswan/ipsec_tunnel.h>
+#include <openswan/ipsec_param.h>
 
 #include "sysdep.h"
 #include "constants.h"
@@ -202,7 +206,9 @@ bool nat_traversal_insert_vid(u_int8_t np, pb_stream *outs)
 	    if (r) r = out_vid(ISAKMP_NEXT_VID, outs, VID_NATT_IETF_05);
 	    if (r) r = out_vid(ISAKMP_NEXT_VID, outs, VID_NATT_IETF_03);
 	    if (r) r = out_vid(ISAKMP_NEXT_VID, outs, VID_NATT_IETF_02_N);
-	    if (r) r = out_vid(np, outs, VID_NATT_IETF_02);
+	    if (r)
+		r = out_vid(nat_traversal_support_non_ike ? ISAKMP_NEXT_VID : np,
+			outs, VID_NATT_IETF_02);
 	}
 	if (nat_traversal_support_non_ike) {
 	    if (r) r = out_vid(np, outs, VID_NATT_IETF_00);
@@ -664,14 +670,52 @@ void nat_traversal_show_result (u_int32_t nt, u_int16_t sport)
 
 int nat_traversal_espinudp_socket (int sk, const char *fam, u_int32_t type)
 {
-	int r;
-	r = setsockopt(sk, SOL_UDP, UDP_ESPINUDP, &type, sizeof(type));
-	if ((r<0) && (errno == ENOPROTOOPT)) {
+	int r = 0;
+	static enum { auto_style, new_style, old_style } style = auto_style;
+
+	if (style == auto_style || style == new_style) {
+		struct ifreq ifr;
+		int *fdp = (int *) &ifr.ifr_data;
+
+		if (style == auto_style)
+			loglog(RC_LOG_SERIOUS, "NAT-Traversal: Trying new style NAT-T");
+
+		memset(&ifr, 0, sizeof(ifr));
+		strcpy(ifr.ifr_name, "ipsec0");
+		fdp[0] = sk;
+		fdp[1] = type;
+		r = ioctl(sk, IPSEC_UDP_ENCAP_CONVERT, &ifr);
+		if (r == -1)
+			loglog(RC_LOG_SERIOUS,
+				   "NAT-Traversal: ESPINUDP(%d) setup failed for "
+				   "new style NAT-T family %s (errno=%d)"
+				   , type, fam, errno);
+		else
+			style = new_style;
+	}
+
+	if (style == auto_style || style == old_style) {
+
+		if (style == auto_style)
+			loglog(RC_LOG_SERIOUS, "NAT-Traversal: Trying old style NAT-T");
+
+		r = setsockopt(sk, SOL_UDP, UDP_ESPINUDP, &type, sizeof(type));
+		if (r == -1)
+			loglog(RC_LOG_SERIOUS,
+				   "NAT-Traversal: ESPINUDP(%d) setup failed for "
+				   "old style NAT-T family %s (errno=%d)"
+				   , type, fam, errno);
+		else
+			style = old_style;
+	}
+
+	if (r == -1 && (errno == ENOPROTOOPT)) {
 		loglog(RC_LOG_SERIOUS,
 		       "NAT-Traversal: ESPINUDP(%d) not supported by kernel for family %s"
 		       , type, fam);
 		disable_nat_traversal(type);
 	}
+
 	return r;
 }
 
@@ -814,8 +858,10 @@ static void nat_traversal_find_new_mapp_state (struct state *st, void *data)
 {
 	struct _new_mapp_nfo *nfo = (struct _new_mapp_nfo *)data;
 
-	if(st->st_serialno == nfo->st->st_clonedfrom ||
-	   st->st_clonedfrom == nfo->st->st_clonedfrom) {
+	if((!nfo->st->st_clonedfrom &&
+	    (st->st_serialno == nfo->st->st_clonedfrom &&
+	     st->st_clonedfrom == nfo->st->st_clonedfrom)) ||
+	   st->st_serialno == nfo->st->st_serialno) {
 		char b1[ADDRTOT_BUF];
 		char b2[ADDRTOT_BUF];
 		struct connection *c = st->st_connection;
