@@ -1,7 +1,11 @@
 /* Pluto main program
  * Copyright (C) 1997      Angelos D. Keromytis.
  * Copyright (C) 1998-2001 D. Hugh Redelmeier.
- * Copyright (C) 2003-2004 Xelerance Corporation
+ * Copyright (C) 2003-2008 Michael C Richardson <mcr@xelerance.com> 
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com> 
+ * Copyright (C) 2007 Ken Bantoft <ken@xelerance.com>
+ * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -15,7 +19,7 @@
  *
  * Modifications to use OCF interface written by
  * Daniel Djamaludin <danield@cyberguard.com>
- * Copyright (C) 2004-2005 Intel Corporation.  All Rights Reserved.
+ * Copyright (C) 2004-2005 Intel Corporation. 
  *
  */
 
@@ -85,14 +89,20 @@
 #include "tpm/tpm.h"
 #endif
 
-#ifdef HAVE_OCF
-#include "ocf_pk.h"
-#endif
+#include "oswcrypto.h"
 
 #ifndef IPSECDIR
 #define IPSECDIR "/etc/ipsec.d"
 #endif
 
+#ifdef HAVE_LIBNSS
+# include <nss.h>
+# include <nspr.h>
+# ifdef FIPS_CHECK
+#  include <fipscheck.h>
+# endif
+#endif
+
 const char *ctlbase = "/var/run/pluto";
 
 openswan_passert_fail_t openswan_passert_fail = passert_fail;
@@ -146,12 +156,14 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--debug-raw]"
 	    " [--debug-crypt]"
+	    " [--debug-crypto]"
 	    " [--debug-parsing]"
 	    " [--debug-emitting]"
 	    " \\\n\t"
 	    "[--debug-control]"
 	    "[--debug-lifecycle]"
 	    " [--debug-klips]"
+	    " [--debug-netkey]"
 	    " [--debug-x509]"
 	    " [--debug-dns]"
 	    " [--debug-oppo]"
@@ -368,11 +380,13 @@ main(int argc, char **argv)
 
 	    { "debug-raw", no_argument, NULL, DBG_RAW + DBG_OFFSET },
 	    { "debug-crypt", no_argument, NULL, DBG_CRYPT + DBG_OFFSET },
+	    { "debug-crypto", no_argument, NULL, DBG_CRYPT + DBG_OFFSET },
 	    { "debug-parsing", no_argument, NULL, DBG_PARSING + DBG_OFFSET },
 	    { "debug-emitting", no_argument, NULL, DBG_EMITTING + DBG_OFFSET },
 	    { "debug-control", no_argument, NULL, DBG_CONTROL + DBG_OFFSET },
 	    { "debug-lifecycle", no_argument, NULL, DBG_LIFECYCLE + DBG_OFFSET },
 	    { "debug-klips", no_argument, NULL, DBG_KLIPS + DBG_OFFSET },
+	    { "debug-netkey", no_argument, NULL, DBG_NETKEY + DBG_OFFSET },
 	    { "debug-dns", no_argument, NULL, DBG_DNS + DBG_OFFSET },
 	    { "debug-oppo", no_argument, NULL, DBG_OPPO + DBG_OFFSET },
 	    { "debug-oppoinfo", no_argument, NULL, DBG_OPPOINFO + DBG_OFFSET },
@@ -736,21 +750,47 @@ main(int argc, char **argv)
 
 	/* make sure that stdin, stdout, stderr are reserved */
 	if (open("/dev/null", O_RDONLY) != 0)
-	    abort();
+	    osw_abort();
 	if (dup2(0, 1) != 1)
-	    abort();
+	    osw_abort();
 	if (!log_to_stderr && dup2(0, 2) != 2)
-	    abort();
+	    osw_abort();
     }
 
     init_constants();
     pluto_init_log();
 
+#ifdef HAVE_LIBNSS
+	char buf[100];
+	snprintf(buf, sizeof(buf), "sql:%s",oco->confddir);
+	loglog(RC_LOG_SERIOUS,"nss directory plutomain: %s",buf);
+	SECStatus nss_init_status= NSS_InitReadWrite(buf);
+	if (nss_init_status != SECSuccess) {
+	    loglog(RC_LOG_SERIOUS, "NSS initialization failed (err %d)\n", PR_GetError());
+        exit_pluto(10);
+	} else {
+	    loglog(RC_LOG_SERIOUS, "NSS Initialized");
+	    PK11_SetPasswordFunc(getNSSPassword);
+#ifdef FIPS_CHECK
+	    if (Pluto_IsFIPS() && !FIPSCHECK_verify(NULL, NULL)) {
+		loglog(RC_LOG_SERIOUS, "FIPS integrity verification test failed");
+		exit_pluto(10);
+	    }
+#endif
+      }
+#endif
+
     /* Note: some scripts may look for this exact message -- don't change
      * ipsec barf was one, but it no longer does.
      */
     {
 #ifdef PLUTO_SENDS_VENDORID
+# ifdef HAVE_LIBNSS
+	if(PK11_IsFIPS()) {
+	openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
+		, ipsec_version_code() , compile_time_interop_options, getpid());
+	} else {
+# endif
         const char *v = init_pluto_vendorid();
 	const char *vc = ipsec_version_code();
 
@@ -775,6 +815,11 @@ main(int argc, char **argv)
 	     */
 	    openswan_log("@(#) built on "__DATE__":" __TIME__ " by " BUILDER);
 	}
+#ifdef HAVE_LIBNSS
+#ifdef PLUTO_SENDS_VENDORID
+	}
+#endif
+#endif
 
 #if defined(USE_1DES)
 	openswan_log("WARNING: 1DES is enabled");
@@ -799,9 +844,7 @@ main(int argc, char **argv)
     init_connections();
     init_crypto();
     init_crypto_helpers(nhelpers);
-#ifdef HAVE_OCF
-    load_cryptodev();
-#endif
+    load_oswcrypto();
     init_demux();
     init_kernel();
     init_adns();
@@ -829,6 +872,11 @@ main(int argc, char **argv)
     /* loading attribute certificates (experimental) */
     load_acerts();
 
+#ifdef HAVE_LIBNSS
+    /*Loading CA certs from NSS DB*/
+    load_authcerts_from_nss("CA cert",  AUTH_CA);
+#endif
+
     daily_log_event();
     call_server();
     return -1;	/* Shouldn't ever reach this */
@@ -871,6 +919,9 @@ exit_pluto(int status)
     free_ifaces();          /* free interface list from memory */
     stop_adns();            /* Stop async DNS process (if running) */
     free_md_pool();         /* free the md pool */
+#ifdef HAVE_LIBNSS
+    NSS_Shutdown();
+#endif
     delete_lock();          /* delete any lock files */
     close_log();            /* close the logfiles */
     exit(status);           /* exit, with our error code */
