@@ -1,6 +1,9 @@
 /* whack communicating routines
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
+ * Copyright (C) 2003-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,8 +14,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * RCSID $Id: rcv_whack.c,v 1.119 2005/10/02 22:30:12 mcr Exp $
  */
 
 #include <stdio.h>
@@ -452,7 +453,10 @@ void whack_process(int whackfd, struct whack_message msg)
     if (msg.whack_reread & REREAD_CACERTS)
     {
 	load_authcerts("CA cert", oco->cacerts_dir, AUTH_CA);
-    }
+#ifdef HAVE_LIBNSS
+       load_authcerts_from_nss("CA cert", AUTH_CA);
+#endif
+     }
 
     if (msg.whack_reread & REREAD_AACERTS)
     {
