@@ -1,6 +1,10 @@
 /* state and event objects
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
+ * Copyright (C) 2003-2008 Michael C Richardson <mcr@xelerance.com> 
+ * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com> 
+ * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +16,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.h,v 1.100 2005/08/05 19:16:49 mcr Exp $
  */
 
 #ifndef _STATE_H
@@ -25,6 +28,11 @@
 #include <gmp.h>    /* GNU MP library */
 #include "quirks.h"
 
+#ifdef HAVE_LIBNSS
+# include <nss.h>
+# include <pk11pub.h>
+#endif
+
 /* Message ID mechanism.
  *
  * A Message ID is contained in each IKE message header.
@@ -265,9 +273,21 @@ struct state
     u_int16_t          st_peeruserport;
 
     /* end of symmetric stuff */
+
+    /* Support quirky feature of Phase 1 ID payload for peer
+     * We don't support this wart for ourselves.
+     * Currently used in Aggressive mode for interop.
+     */
+    u_int8_t           st_peeridentity_protocol;
+    u_int16_t          st_peeridentity_port;
+
     u_int8_t           st_sec_in_use;      /* bool: does st_sec hold a value */
     MP_INT             st_sec;             /* Our local secret value */
     chunk_t            st_sec_chunk;       /* copy of above */
+#ifdef HAVE_LIBNSS
+    /*DH public key*/
+    chunk_t            pubk;
+#endif
 
     chunk_t            st_shared;              /* Derived shared secret
                                                 * Note: during Quick Mode,
@@ -412,10 +432,6 @@ extern void show_states_status(void);
 void for_each_state(void *(f)(struct state *, void *data), void *data);
 #endif
 
-#if 1
-void for_each_state(void *(f)(struct state *, void *data), void *data);
-#endif
-
 extern void find_my_cpi_gap(cpi_t *latest_cpi, cpi_t *first_busy_cpi);
 extern ipsec_spi_t uniquify_his_cpi(ipsec_spi_t cpi, struct state *st);
 extern void fmt_state(struct state *st, time_t n
@@ -436,6 +452,7 @@ extern void delete_states_dead_interfaces(void);
  */
 #ifdef HAVE_STATSD
 #define refresh_state(st) log_state(st, st->st_state)
+#define fake_state(st,new_state) log_state(st, new_state)
 #define change_state(st,new_state) \
 	do { \
 		if ((new_state) != (st)->st_state) { \
@@ -445,6 +462,7 @@ extern void delete_states_dead_interfaces(void);
 	   } while(0)
 #else
 #define refresh_state(st) /* do nothing */
+#define fake_state(st,new_state) /* do nothing */
 #define change_state(st, new_state) do { (st)->st_state=(new_state); } while(0)
 #endif
 
